from sirio.business_object import BusinessObject, Object
import requests


class SirioReport:
    
    def __init__(self, hostSirio: str, separatorBindId: str = '%'):
        self.hostSirio = hostSirio
        self.separatorBindId = separatorBindId
        self.presignedUrlReport = None

        def RenderReport(self, templateName:str, domain: str, businessObject: BusinessObject) -> bool:
            esito = False
            url = f'{self.hostSirio}/pdf/{domain}/report'
            urlParams = f'{self.hostSirio}/pdf/{domain}/template/{templateName}/params'
            responseParams = requests.get(urlParams)
            if responseParams.status_code == 200:
                params = responseParams.json()  # Supponiamo che sia una lista di stringhe
                dizionario = {}

                for bindId in params:
                    parti = bindId.split(self.separatorBindId, 1)
                    valore = businessObject.getValue(bind=parti[0], id=parti[1])
                    dizionario[bindId] = valore

                requestReport = {
                        "templateName": templateName,
                        "data": dizionario
                }
                response = requests.post(url=url, json=requestReport)
                if response.status_code == 200:
                    self.presignedUrlReport = response.text
                    
                    print(f'RenderReport: {response.text}')
                    esito = True
            else:
                print(f"Errore nella richiesta: {responseParams.status_code}")
            return esito
        
        def getPresignedUrlReport(self) -> str:
            """
            Restituisce la presigned URL del report generato.

            :return: La presigned URL del report.
            :raises Exception: Se il report non è stato generato o se la presigned URL non è disponibile.
            """
            if self.presignedUrlReport is None:
                raise Exception("Il report non è stato generato o la presigned URL non è disponibile.")
            return self.presignedUrlReport
        
        def getReportStream(self):
            """
            Restituisce uno stream del report generato.

            :return: Uno stream (BytesIO) contenente il report.
            :raises Exception: Se il report non è stato generato o se la presigned URL non è disponibile.
            """
            if self.presignedUrlReport is None:
                raise Exception("Il report non è stato generato o la presigned URL non è disponibile.")
            
            return self.__get_stream_from_presigned_url(self.presignedUrlReport)

        def __get_stream_from_presigned_url(self, presigned_url):
            """
            Scarica un file da una presigned URL e lo restituisce come stream (BytesIO).

            :param presigned_url: La presigned URL da cui scaricare il file.
            :return: Uno stream (BytesIO) contenente il contenuto del file.
            :raises Exception: Se la richiesta fallisce.
            """
            response = requests.get(presigned_url, stream=True)

            if response.status_code == 200:
                # Carica tutto il contenuto in memoria in uno stream
                file_stream = io.BytesIO(response.content)
                file_stream.seek(0)  # Si assicura che lo stream parta dall'inizio
                return file_stream
            else:
                raise Exception(f"Errore nel download: {response.status_code}")


from typing import List
from sirio.event import ObjEvent
import requests
import json
            
class Message:
    role: str
    content: str

class RequestAiBatch:
    key: str
    idSync: str = None
    sirioDomain: str = None
    application: str
    messages: List[Message]

class SirioAiBatch:
    
    def __init__(self, hostOpenAiBatchRequest: str):
        self.hostOpenAiBatchRequest = hostOpenAiBatchRequest

        def requestAi(self, requestAi: RequestAiBatch) -> str:
            
            url = f'{self.hostOpenAiBatchRequest}/batch/request'
            print(f'URL: {url}')
            response = requests.post(url=url, json=requestAi)
            if response.status_code == 200:
                print(f'Invovazione per openai-batch-request, domain: {requestAi.domain} - idSync: {requestAi.idSync}: response: {response.text}')
                id_openai_batch_request = response.text.replace('"', '')
                print(f'id_openai_batch_request: {id_openai_batch_request}')
                return id_openai_batch_request
            else:
                print(f'Problemi con invocazione openai-batch-request, domain: {requestAi.domain} - idSync: {requestAi.idSync} - status: {response.status_code}: {response.text}')
                raise Exception(f'Problema di invocazione objEvent.domain --> status: {response.status_code}: {response.text}')
        



from dataclasses import dataclass
from typing import Callable, List, Tuple, Any

from sirio.business_object import BusinessObject

Result = Tuple[List[str], Any] | None

Extractor = Callable[[BusinessObject], Result]


class _Filtered:
    pass


_FILTERED = _Filtered()


@dataclass(frozen=True)
class Value:
    f: Callable[[Any], Any]

    def map(self, g: Callable[[Any], Any]) -> "Value":
        def map_function(x):
            value = self.f(x)
            if value is _FILTERED:
                return _FILTERED
            return g(value)

        return Value(map_function)

    def filter(self, p: Callable[[Any], bool]) -> "Value":
        def filtered_function(x):
            value = self.f(x)
            if value is _FILTERED:
                return _FILTERED
            return value if p(value) else _FILTERED

        return Value(filtered_function)

    def to(self, first: str, *others: str) -> Extractor:
        def extractor(x):
            value = self.f(x)
            return None if value is _FILTERED else (_merge_keys(first, others), value)

        return extractor


def extract_business_key() -> Value:
    return Value(lambda bo: bo.businessKey)


def extract_internal_business_key() -> Value:
    return Value(lambda bo: bo.internalBusinessKey)


def extract_bind_id(bind: str, id: str) -> Value:
    return Value(lambda bo: bo.getValue(bind, id))


def _merge_keys(first: str, others: Tuple[str, ...]) -> List[str]:
    return [first] + list(others)


def extract(bo: Any, *extractors: Extractor) -> dict:
    result = {}

    for extractor in extractors:
        r = extractor(bo)
        if r:
            keys, value = r
            _merge(result, keys, value)

    return result


def _merge(result: dict[str, Any], keys: List[str], value: Any) -> None:
    current = result
    for key in keys[:-1]:
        if key not in current:
            current[key] = {}
        current = current[key]
    current[keys[-1]] = value
